@@ -20,14 +20,15 @@
 #define e5bd51be_DOM_HPP_
 
 #include "state.hpp"
-#include "string_view.hpp"
 #include "xml.hpp"
 #include "stream.hpp"
 #include <unicode/utext.h>
 #include <unicode/regex.h>
 #include <libxml/tree.h>
+#include <string_view>
 #include <array>
 #include <deque>
+#include <map>
 
 namespace Transfuse {
 
@@ -144,17 +145,13 @@ struct DOM {
 	icu::RegexMatcher rx_blank_tail;
 	icu::RegexMatcher rx_any_alnum;
 
-	xmlChars tags_prot; // Protected tags
-	xmlChars tags_prot_inline; // Protected inline tags
-	xmlChars tags_raw; // Tags with raw CDATA contents that should not be XML-mangled
-	xmlChars tags_inline; // Inline tags
-	xmlChars tags_parents_allow; // If set, only extract children of these tags
-	xmlChars tags_parents_direct; // Used for TTX <df>?
-	xmlChars tag_attrs; // Attributes that should also be extracted
+	std::map<std::string_view, xmlChars> tags;
 
 	DOM(State&, xmlDocPtr);
 	~DOM();
 
+	void cmdline_tags();
+
 	void save_spaces(xmlNodePtr, size_t);
 	void save_spaces() {
 		save_spaces(reinterpret_cast<xmlNodePtr>(xml.get()), 0);
@@ -189,10 +186,10 @@ struct DOM {
 		return rv;
 	}
 
-	void extract_blocks(xmlString&, xmlNodePtr, size_t, bool txt = false);
+	void extract_blocks(xmlString&, xmlNodePtr, size_t, bool txt = false, bool header = false);
 	xmlString extract_blocks() {
 		xmlString rv;
-		stream->stream_header(rv, state.tmpdir);
+		stream->stream_header(rv, state.settings->tmpdir);
 		blocks = 0;
 		extract_blocks(rv, reinterpret_cast<xmlNodePtr>(xml.get()), 0);
 		return rv;
