@@ -121,6 +121,7 @@ void docx_merge_wt(State& state, xmlDocPtr xml) {
 			}
 			xmlUnlinkNode(bp);
 			xmlFreeNode(bp);
+			ts->nodesetval->nodeTab[j] = nullptr;
 		}
 		xmlXPathFreeObject(ts);
 
@@ -139,6 +140,10 @@ void docx_merge_wt(State& state, xmlDocPtr xml) {
 		for (int j = 0; j < hs->nodesetval->nodeNr; ++j) {
 			auto node = hs->nodesetval->nodeTab[j];
 			auto text = node->children;
+			// Don't merge if this hyperlink has other data, such as TOCs do
+			if (text->next) {
+				continue;
+			}
 			xmlUnlinkNode(text);
 			xmlAddPrevSibling(node, text);
 
@@ -178,16 +183,54 @@ std::unique_ptr<DOM> extract_docx(State& state) {
 	}
 
 	zip_stat_t stat{};
-	if (zip_stat(zip, "word/document.xml", 0, &stat) != 0) {
-		throw std::runtime_error("DOCX did not have word/document.xml");
+
+	// DOCX allows changing the name of the main document, so handle that if word/document.xml doesn't exist
+	std::string docname{"word/document.xml"};
+	if (zip_stat(zip, docname.c_str(), 0, &stat) != 0) {
+		if (zip_stat(zip, "[Content_Types].xml", 0, &stat) != 0) {
+			throw std::runtime_error("DOCX did not have [Content_Types].xml");
+		}
+		if (stat.size == 0) {
+			throw std::runtime_error("DOCX [Content_Types].xml was empty");
+		}
+
+		auto zf = zip_fopen_index(zip, stat.index, 0);
+		if (zf == nullptr) {
+			throw std::runtime_error("Could not open DOCX [Content_Types].xml");
+		}
+
+		std::string ctypes(stat.size, 0);
+		zip_fread(zf, &ctypes[0], stat.size);
+		zip_fclose(zf);
+
+		auto off = ctypes.find(".xml\" ContentType=\"application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml\"");
+		if (off != std::string::npos) {
+			auto nb = ctypes.rfind('"', off) + 1;
+			auto ne = ctypes.find('"', off);
+			docname.assign(ctypes.begin() + nb, ctypes.begin() + ne);
+		}
+		else if ((off = ctypes.find(" ContentType=\"application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml\" PartName=\"")) != std::string::npos) {
+			auto nb = ctypes.find("PartName=\"", off) + 10;
+			auto ne = ctypes.find('"', nb);
+			docname.assign(ctypes.begin() + nb, ctypes.begin() + ne);
+		}
+		if (docname[0] == '/') {
+			docname.erase(docname.begin());
+		}
+	}
+
+	state.info("docx-document-main", docname);
+
+	if (zip_stat(zip, docname.c_str(), 0, &stat) != 0) {
+		throw std::runtime_error(concat("DOCX did not have main document ", docname));
 	}
 	if (stat.size == 0) {
-		throw std::runtime_error("DOCX document.xml was empty");
+		throw std::runtime_error(concat("DOCX main document ", docname, " was empty"));
 	}
 
 	auto zf = zip_fopen_index(zip, stat.index, 0);
 	if (zf == nullptr) {
-		throw std::runtime_error("Could not open DOCX document.xml");
+		throw std::runtime_error(concat("Could not open DOCX main document ", docname));
 	}
 
 	std::string data(stat.size, 0);
@@ -225,8 +268,38 @@ std::unique_ptr<DOM> extract_docx(State& state) {
 	udata.findAndReplace("<w:rFonts></w:rFonts>", "");
 	udata.findAndReplace("<w:rPr></w:rPr>", "");
 	udata.findAndReplace("<w:softHyphen/>", "");
+	udata.findAndReplace("<w:br/>", "<w:t>\n</w:t>");
+	udata.findAndReplace("<w:cr/>", "<w:t>\n</w:t>");
+	udata.findAndReplace("<w:noBreakHyphen/>", "<w:t>-</w:t>");
 
-	rx_replaceAll(R"X(</w:t>([^<>]+?)<w:t(?=[ >])[^>]*>)X", "", udata, tmp);
+	rx_replaceAll(R"X(</w:t>([^<>]*?)<w:t(?=[ >])[^>]*>)X", "", udata, tmp);
+
+	// Move <w:tab> to its very own <w:r> so it doesn't interfere with <w:t> merging or style hashing
+	UErrorCode status = U_ZERO_ERROR;
+	RegexMatcher rx_wr(R"X(<w:r(?=[ >])[^>]*>.*?</w:r>)X", 0, status);
+
+	tmp.remove();
+	rx_wr.reset(udata);
+	int32_t last = 0;
+	while (rx_wr.find(last, status)) {
+		auto mb = rx_wr.start(0, status);
+		auto me = rx_wr.end(0, status);
+
+		tmp.append(udata, last, mb - last);
+		int32_t tab = 0;
+		if ((tab = udata.indexOf("<w:tab/><w:t>", mb, me - mb)) != -1) {
+			tmp.append(udata, mb, tab - mb);
+			tmp.append("<w:tab/></w:r>");
+			tmp.append(udata, mb, tab - mb);
+			tmp.append(udata, tab + 8, me - tab - 8);
+		}
+		else {
+			tmp.append(udata, mb, me - mb);
+		}
+		last = me;
+	}
+	tmp.append(udata, last, udata.length() - last);
+	std::swap(tmp, udata);
 
 	auto xml = xmlReadMemory(reinterpret_cast<const char*>(udata.getTerminatedBuffer()), SI(SZ(udata.length()) * sizeof(UChar)), "document.xml", utf16_native, XML_PARSE_RECOVER | XML_PARSE_NONET);
 	if (xml == nullptr) {
@@ -238,7 +311,8 @@ std::unique_ptr<DOM> extract_docx(State& state) {
 	docx_merge_wt(state, xml);
 
 	auto dom = std::make_unique<DOM>(state, xml);
-	dom->tags_parents_allow = make_xmlChars("tf-text", "w:t");
+	dom->tags[Strs::tags_parents_allow] = make_xmlChars("tf-text", "w:t");
+	dom->cmdline_tags();
 	dom->save_spaces();
 
 	auto buf = xmlBufferCreate();
@@ -274,11 +348,11 @@ std::string inject_docx(DOM& dom) {
 	UnicodeString tmp;
 
 	// DOCX can't have any text outside w:t
-	// Move text from after </w:t></w:r> inside it
-	rx_replaceAll(R"X((</w:t></w:r>)([^<>]+))X", "$2$1", udata, tmp);
+	// Wrap tags around text after </w:t></w:r>, in a way that does not inherit formatting
+	rx_replaceAll(R"X((</w:t></w:r>)([^<>]+))X", "$1<w:r><w:t>$2</w:t></w:r>", udata, tmp);
 
-	// Move text from after </w:t></w:r></w:hyperlink> inside it
-	rx_replaceAll(R"X((</w:t></w:r></w:hyperlink>)([^<>]+))X", "$2$1", udata, tmp);
+	// Ditto for text after </w:t></w:r></w:hyperlink>
+	rx_replaceAll(R"X((</w:t></w:r></w:hyperlink>)([^<>]+))X", "$1<w:r><w:t>$2</w:t></w:r>", udata, tmp);
 
 	// Move text from before <w:r><w:t> inside it
 	rx_replaceAll_expand_21(R"X(([^>])(<w:r(?=[ >])[^>]*>.*?<w:t(?=[ >])[^>]*>))X", udata, tmp);
@@ -300,6 +374,8 @@ std::string inject_docx(DOM& dom) {
 	udata.toUTF8String(data);
 	file_save("injected.xml", data);
 
+	hook_inject(dom.state.settings, "injected.xml");
+
 	fs::copy("original", "injected.docx");
 
 	int e = 0;
@@ -313,8 +389,9 @@ std::string inject_docx(DOM& dom) {
 		throw std::runtime_error("Could not open injected.xml");
 	}
 
-	if (zip_file_add(zip, "word/document.xml", src, ZIP_FL_OVERWRITE) < 0) {
-		throw std::runtime_error("Could not replace word/document.xml");
+	auto docname = dom.state.info("docx-document-main");
+	if (zip_file_add(zip, docname.c_str(), src, ZIP_FL_OVERWRITE) < 0) {
+		throw std::runtime_error(concat("Could not replace main document ", docname));
 	}
 
 	zip_close(zip);
