@@ -30,7 +30,11 @@ using namespace icu;
 
 namespace Transfuse {
 
-std::pair<fs::path,std::string> inject(fs::path tmpdir, std::istream& in, Stream stream) {
+std::pair<fs::path,std::string> inject(Settings& settings) {
+	fs::path& tmpdir = settings.tmpdir;
+	std::istream& in = *settings.in;
+	Stream& stream = settings.stream;
+
 	std::ios::sync_with_stdio(false);
 	in.tie(nullptr);
 
@@ -41,24 +45,29 @@ std::pair<fs::path,std::string> inject(fs::path tmpdir, std::istream& in, Stream
 	std::unique_ptr<StreamBase> sformat;
 
 	std::string buffer;
-	std::getline(in, buffer);
+	while (buffer.empty()) {
+		std::getline(in, buffer);
+	}
 
 	if (stream == Streams::detect) {
 		if (buffer.find("[transfuse:") != std::string::npos) {
-			sformat.reset(new ApertiumStream);
+			sformat.reset(new ApertiumStream(&settings));
 		}
 		else if (buffer.find("<STREAMCMD:TRANSFUSE:") != std::string::npos) {
-			sformat.reset(new VISLStream);
+			sformat.reset(new VISLStream(&settings));
 		}
 		else {
 			throw std::runtime_error("Could not detect input stream format");
 		}
 	}
 	else if (stream == Streams::apertium) {
-		sformat.reset(new ApertiumStream);
+		sformat.reset(new ApertiumStream(&settings));
+	}
+	else if (stream == Streams::cg) {
+		sformat.reset(new CGStream(&settings));
 	}
 	else {
-		sformat.reset(new VISLStream);
+		sformat.reset(new VISLStream(&settings));
 	}
 
 	if (tmpdir.empty()) {
@@ -89,8 +98,16 @@ std::pair<fs::path,std::string> inject(fs::path tmpdir, std::istream& in, Stream
 		if (bid.empty()) {
 			continue;
 		}
-		reduce_ws(buffer);
-		assign_xml(tmp_b, buffer);
+		if (settings.opt_inject_raw) {
+			tmp_b += buffer;
+		}
+		else if (stream != Streams::cg) {
+			reduce_ws(buffer);
+			assign_xml(tmp_b, buffer);
+		}
+		else {
+			assign_xml(tmp_b, buffer, true);
+		}
 		buffer.swap(tmp_b);
 
 		tmp_b = TFB_OPEN_B;
@@ -137,7 +154,7 @@ std::pair<fs::path,std::string> inject(fs::path tmpdir, std::istream& in, Stream
 
 	cleanup_styles(content);
 
-	State state(fs::current_path(), true);
+	State state(&settings, true);
 
 	UText tmp_ut = UTEXT_INITIALIZER;
 	UErrorCode status = U_ZERO_ERROR;
@@ -258,6 +275,12 @@ std::pair<fs::path,std::string> inject(fs::path tmpdir, std::istream& in, Stream
 	else if (format == "text") {
 		fname = inject_text(*dom);
 	}
+	else if (format == "tei") {
+		fname = inject_tei(*dom);
+	}
+	else if (format == "line") {
+		fname = inject_text(*dom, true);
+	}
 
 	return {tmpdir, fname};
 }
