@@ -9,7 +9,7 @@
 #ifndef e5bd51be_OPTIONS_HPP_
 #define e5bd51be_OPTIONS_HPP_
 
-#include "string_view.hpp"
+#include <string_view>
 #include <utility>
 #include <array>
 #include <algorithm>
@@ -24,6 +24,9 @@ enum ArgType : uint8_t {
 	ARG_NO, // Does not take any arguments
 	ARG_OPT, // Optionally takes an argument
 	ARG_REQ, // Requires an argument
+	TYPE_SPACER,
+	TYPE_TEXT,
+	TYPE_FINAL,
 };
 
 struct Option {
@@ -41,6 +44,11 @@ struct Option {
 	  , desc(desc)
 	{}
 
+	Option(ArgType arg = TYPE_SPACER, std::string_view desc = {})
+	  : arg(arg)
+	  , desc(desc)
+	{}
+
 	Option(char opt, std::string_view longopt = {}, std::string_view desc = {})
 	  : opt(opt)
 	  , longopt(longopt)
@@ -56,11 +64,14 @@ struct Option {
 
 using O = Option;
 
+inline Option text(std::string_view txt) {
+	return { TYPE_TEXT, txt };
+}
 inline Option spacer() {
-	return { 0, {}, ARG_OPT };
+	return { TYPE_SPACER };
 }
 inline Option final() {
-	return { 0, {}, ARG_REQ };
+	return { TYPE_FINAL };
 }
 
 inline int _parse(int argc, char* argv[], Option* opts, size_t cnt) {
@@ -211,69 +222,65 @@ struct Options {
 	std::string explain() {
 		std::string rv;
 
-		size_t sz = 0;
-		size_t longest = 0;
-		for (size_t i = 0; i < N; i++) {
-			sz += 1;
-			if (!opts[i].desc.empty()) {
-				sz += 6 + opts[i].desc.size(); // At least space, dash, letter, space, description, newline
-				if (!opts[i].longopt.empty()) {
-					size_t len = opts[i].longopt.size();
-					sz += 4 + len; // At least comma, space, 2 dashes, long name
-					longest = std::max(longest, len);
-				}
-				sz += 3; // Wild guess at average padding amount
-			}
-			else if (!opts[i].opt && opts[i].longopt.empty() && opts[i].arg == ARG_REQ) {
-				// Null option with required arg is final()
-				break;
-			}
-		}
-		// Preallocate the guessed size
-		rv.reserve(sz);
-
-		for (size_t i = 0; i < N; i++) {
-			if (opts[i].desc.empty()) {
-				if (opts[i].opt || !opts[i].longopt.empty()) {
-					// Only options with descriptions are rendered
-				}
-				else if (opts[i].arg == ARG_REQ) {
-					// Null option with required arg is final()
+		for (size_t at = 0; at < N;) {
+			size_t longest = 0;
+			for (size_t i = at; i < N; i++) {
+				if (opts[i].arg == TYPE_FINAL || opts[i].arg == TYPE_TEXT) {
 					break;
 				}
-				else {
-					// Null option with optional arg is spacer()
+				if (!opts[i].desc.empty()) {
+					if (!opts[i].longopt.empty()) {
+						size_t len = opts[i].longopt.size();
+						longest = std::max(longest, len);
+					}
+				}
+			}
+
+			for (size_t i = at; i < N; i++) {
+				if (opts[i].arg == TYPE_FINAL) {
+					at = N;
+					break;
+				}
+				if (opts[i].arg == TYPE_SPACER) {
 					rv += '\n';
 				}
-				continue;
-			}
+				if (opts[i].arg == TYPE_TEXT) {
+					at = i + 1;
+					rv += opts[i].desc;
+					rv += '\n';
+					break;
+				}
+				if (opts[i].desc.empty()) {
+					continue;
+				}
 
-			rv += ' ';
-			size_t ldiff = longest;
-			if (opts[i].opt && !opts[i].longopt.empty()) {
-				rv += '-';
-				rv += opts[i].opt;
-				rv += ", --";
-				rv += opts[i].longopt;
-				ldiff -= opts[i].longopt.size();
-			}
-			else if (opts[i].opt) {
-				rv += '-';
-				rv += opts[i].opt;
-				rv += "    ";
-			}
-			else if (!opts[i].longopt.empty()) {
-				rv += "    --";
-				rv += opts[i].longopt;
-				ldiff -= opts[i].longopt.size();
-			}
-
-			while (ldiff--) {
 				rv += ' ';
+				size_t ldiff = longest;
+				if (opts[i].opt && !opts[i].longopt.empty()) {
+					rv += '-';
+					rv += opts[i].opt;
+					rv += ", --";
+					rv += opts[i].longopt;
+					ldiff -= opts[i].longopt.size();
+				}
+				else if (opts[i].opt) {
+					rv += '-';
+					rv += opts[i].opt;
+					rv += "    ";
+				}
+				else if (!opts[i].longopt.empty()) {
+					rv += "    --";
+					rv += opts[i].longopt;
+					ldiff -= opts[i].longopt.size();
+				}
+
+				while (ldiff--) {
+					rv += ' ';
+				}
+				rv += "  ";
+				rv += opts[i].desc;
+				rv += '\n';
 			}
-			rv += "  ";
-			rv += opts[i].desc;
-			rv += '\n';
 		}
 
 		return rv;
