@@ -19,10 +19,12 @@
 #ifndef e5bd51be_SHARED_HPP_
 #define e5bd51be_SHARED_HPP_
 
-#include "string_view.hpp"
 #include "filesystem.hpp"
 #include <unicode/unistr.h>
+#include <map>
+#include <set>
 #include <string>
+#include <string_view>
 #include <fstream>
 #include <algorithm>
 #include <cctype>
@@ -46,6 +48,8 @@ constexpr auto XML_ENC_UC = static_cast<UChar>(u'\uE014');
 #define TFU_OPEN     "\xee\x80\xa2" /* \uE022 */
 #define TFU_CLOSE    "\xee\x80\xa3" /* \uE023 */
 
+#define TF_CURVED_PARAGRAPH "\xe2\x9d\xa1" /* \u2761 ❡ CURVED STEM PARAGRAPH SIGN ORNAMENT */
+
 #if defined(ARCH_BIG_ENDIAN)
 	const std::string_view utf16_bom{ "\xfe\xff" };
 	const auto utf16_native = "UTF-16BE";
@@ -112,13 +116,6 @@ namespace details {
 	inline void _concat(std::string&) {
 	}
 
-	// ToDo: C++17 renders this function obsolete
-	template<typename... Args>
-	inline void _concat(std::string& msg, std::string_view t, Args... args) {
-		msg += t;
-		_concat(msg, args...);
-	}
-
 	template<typename T, typename... Args>
 	inline void _concat(std::string& msg, const T& t, Args... args) {
 		msg.append(t);
@@ -267,9 +264,56 @@ inline void file_save(fs::path fn, const icu::UnicodeString& data, bool bom = tr
 	}
 }
 
-std::string detect_encoding(std::string_view);
+std::string detect_encoding(std::string_view data);
 
-icu::UnicodeString to_ustring(std::string_view, std::string_view);
+icu::UnicodeString to_ustring(std::string_view data, std::string_view encoding);
+
+namespace Streams {
+	const std::string_view detect{ "detect" };
+	const std::string_view apertium{ "apertium" };
+	const std::string_view visl{ "visl" };
+	const std::string_view cg{ "cg" };
+}
+using Stream = std::string_view;
+
+namespace Strs {
+	constexpr std::string_view tags_prot{ "tags-prot" }; // Protected tags
+	constexpr std::string_view tags_prot_inline{ "tags-prot-inline" }; // Protected inline tags
+	constexpr std::string_view tags_raw{ "tags-raw" }; // Tags with raw CDATA contents that should not be XML-mangled
+	constexpr std::string_view tags_inline{ "tags-inline" }; // Inline tags
+	constexpr std::string_view tags_parents_allow{ "tags-parents-allow" }; // If set, only extract children of these tags
+	constexpr std::string_view tags_parents_direct{ "tags-parents-direct" }; // Used for TTX <df>?
+	constexpr std::string_view tag_attrs{ "tag-attrs" }; // Tags that should append ❡ (U+2761)
+	constexpr std::string_view tags_headers{ "tags-headers" }; // Attributes that should also be extracted
+	constexpr std::string_view attrs_headers{ "attrs-headers" }; // Attributes that should append ❡ (U+2761)
+}
+inline constexpr auto maybe_tags = { Strs::tags_prot, Strs::tags_prot_inline, Strs::tags_raw, Strs::tags_inline, Strs::tags_parents_allow, Strs::tags_parents_direct, Strs::tag_attrs, Strs::tags_headers, Strs::attrs_headers };
+
+struct Settings {
+	std::string_view mode{ "clean" };
+	std::string_view format{ "auto" };
+	Stream stream{ Streams::detect };
+
+	fs::path tmpdir;
+	fs::path infile;
+	std::istream* in = nullptr;
+	std::unique_ptr<std::istream> _in;
+	std::ostream* out = nullptr;
+	std::unique_ptr<std::ostream> _out;
+
+	bool opt_debug = false;
+	bool opt_keep = false;
+	bool opt_no_keep = false;
+	bool opt_mark_headers = false;
+	bool opt_apertium_n = false;
+	bool opt_inject_raw = false;
+
+	std::string_view hook_inject;
+
+	std::map<std::string_view, std::set<std::string_view>> tags;
+};
+
+void hook_inject(Settings* settings, std::string_view fn);
 
 }
 
