@@ -19,20 +19,19 @@
 #ifndef e5bd51be_STATE_HPP_
 #define e5bd51be_STATE_HPP_
 
-#include "string_view.hpp"
 #include "filesystem.hpp"
 #include "xml.hpp"
+#include "shared.hpp"
 #include <string>
+#include <string_view>
 #include <memory>
 
 namespace Transfuse {
 
 struct State {
-	fs::path tmpdir;
-	bool opt_verbose = false;
-	bool opt_debug = false;
+	Settings* settings;
 
-	State(fs::path, bool ro = false);
+	State(Settings*, bool ro = false);
 	~State();
 
 	void begin();
